#!/bin/bash


echo "$0 $@"  # Print the command line for logging
. utils/parse_options.sh || exit 1;

. cmd.sh
featdir=mfcc_vtln
num_leaves=2500
num_gauss=15000


# train linear vtln
steps/train_lvtln.sh --cmd "$train_cmd" $num_leaves $num_gauss \
  data/train data/lang exp/tri2_ali exp/triVTLN || exit 1
mkdir -p data/train_vtln
cp -r data/train/* data/train_vtln || exit 1
cp exp/triVTLN/final.warp data/train_vtln/spk2warp || exit 1

utils/mkgraph.sh data/lang_test_bg \
  exp/triVTLN exp/triVTLN/graph || exit 1;

nj=10
steps/decode_lvtln.sh --nj $nj --cmd "$decode_cmd" exp/triVTLN/graph data/test exp/triVTLN/decode_test || exit 1
mkdir -p data/test_vtln
cp -r data/test/* data/test_vtln || exit 1
cp exp/triVTLN/decode_test/final.warp data/test_vtln/spk2warp || exit 1



steps/make_mfcc.sh --nj 20 --cmd "$train_cmd" data/test_vtln exp/make_mfcc/test_vtln ${featdir} || exit 1
steps/compute_cmvn_stats.sh data/test_vtln exp/make_mfcc/test_vtln ${featdir} || exit 1
utils/fix_data_dir.sh data/test_vtln || exit 1 # remove segments with problems


steps/align_si.sh --nj 10 --cmd "$train_cmd" \
  data/train_vtln data/lang \
  exp/triVTLN exp/triVTLN_ali || exit 1

steps/train_lda_mllt.sh --cmd "$train_cmd" \
  --splice-opts "--left-context=3 --right-context=3" \
  2500 15000 data/train_vtln \
  data/lang exp/triVTLN_ali exp/tri2b  || exit 1

(
utils/mkgraph.sh data/lang_test_bg \
  exp/tri2b exp/tri2b/graph || exit 1;
steps/decode.sh --nj 10 --cmd "$decode_cmd" \
  exp/tri2b/graph data/test_vtln \
  exp/tri2b/decode_test || exit 1;
) &

steps/align_si.sh --nj 10 --cmd "$train_cmd" \
  data/train_vtln data/lang \
  exp/tri2b exp/tri2b_ali || exit 1

# From 2d system, train 3c which is LDA + MLLT + SAT.
steps/train_sat.sh --cmd "$train_cmd" \
  2500 15000 data/train_vtln \
  data/lang exp/tri2b_ali exp/tri3c || exit 1;

(
utils/mkgraph.sh data/lang_test_bg \
  exp/tri3c exp/tri3c/graph || exit 1;
steps/decode_fmllr.sh --nj 10 --cmd "$decode_cmd" \
  exp/tri3c/graph data/test_vtln \
  exp/tri3c/decode_test || exit 1;
) &
